# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

import numpy as np
from .make import adapted_grid


def plot_convergence(key, values, cb, metric=None, xstart=0, xstop=2, **kwargs):
    import matplotlib.pyplot as plt

    if key in kwargs:
        raise ValueError("Cannot have key=%s when given in kwargs" % key)
    fig, axes = plt.subplots(1, len(values), figsize=(16, 5),
                             sharey=True, gridspec_kw={'wspace': 0})
    scores, grid_sizes = [], []
    if key is None and len(values) != 1:
        raise ValueError("Not considering key")
    for val, ax in zip(values, np.atleast_1d(axes)):
        if key is not None:
            kwargs[key] = val
        grid, results = adapted_grid(xstart, xstop, cb, metric=metric, **kwargs)
        y = results if metric is None else np.array(
            [metric(r) for r in results])
        ax.vlines(grid, 0, 1, transform=ax.get_xaxis_transform(),
                  linestyle='--', color='k', alpha=.3, linewidth=.5)
        ax.plot(grid, y)
        between_x = grid[:-1] + np.diff(grid)/2
        between_y = y[:-1] + np.diff(y)/2
        rbx = cb(between_x)
        ybx = rbx if metric is None else np.array([metric(r) for r in rbx])
        scores.append(np.sum(np.abs(between_y - ybx)))
        grid_sizes.append(grid.size)
    return np.array(scores), grid_sizes



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

import numpy as np
from ..make import adapted_grid
from ._common import g


def test_adapted_grid():
    ag, y = adapted_grid(0, 2, g, (8,)*4)
    assert ag.shape == (32,)
    assert y.shape == (32,)


def test_adapted_grid__performance():
    r = []
    for ga in [(32,), (16,)*2, (8,)*4]:
        grid, y = adapted_grid(0, 2, g, ga)
        bx = grid[:-1] + np.diff(grid)/2
        by = y[:-1] + np.diff(y)/2
        r.append(np.sum(np.abs(by - g(bx))))

    assert np.all(r < [.272, 0.25, .15])
    assert np.all(np.diff(r) < 0)



# Cython (www.cython.org) definition file
cdef extern from "newton_interval.h":
    int get_interval(const double * const arr, int N, const double t) nogil
    int get_interval_from_guess(const double * const arr, int N, double t, int i) nogil
    int check_nan(const double * const, int) nogil
    int check_strict_monotonicity(const double * const, int) nogil
